// $Id: align_primer.cpp 1151 2021-01-29 00:45:32Z ge $
/// \file align_sequence.cpp
/// \brief program for aligning sequences
/// \author Gerald Weber <gweberbh@gmail.com>
/// $Revision: 1151 $

#include "NucleotideSequence.h"
#include "FileParser.h"
#include "Options.h"
#include "AlignSequence.h"
#include <unistd.h>
#include "CPUtime.h"

using namespace gbc;
using namespace std;
namespace gbc_test=gbc_exp;
using namespace gbc_test;

int main(int argc,char *argv[])
  {
#ifndef SVNVERSION
#define SVNVERSION "$Revision: 865$"
#endif
#ifndef LINUXSYSTEM
#define LINUXSYSTEM "UKNOWN"
#endif
#ifndef BUILDTIME
#define BUILDTIME "UKNOWN"
#endif
   
  std::cout << "*** align_primer version=" << std::string(SVNVERSION) << " system=" << std::string(LINUXSYSTEM) << " builddate=" << std::string(BUILDTIME) << std::endl;

  Option<string>  output_name("o","output files basename",OptionMap::necessary);
  Option<string>  genome_name("g","genome fasta file name",OptionMap::necessary);
  Option<string>  target_name("p","primer fasta file name",OptionMap::necessary);
  Option<string>  directions("dir","alignment directions m=main r=main-reverse c=complementary d=complementary-reverse",OptionMap::optional,"rD");
  Option<int>     margin("m","fasta file contig margin",OptionMap::optional,80);
  Option<int>     max_contig("mx","Maximum contig size",OptionMap::optional,100000);
  Option<int>     skip("s","skip a number of nucleotides in genome file",OptionMap::optional,0);
  Option<double>  overlap("ov","Scan overlap",OptionMap::optional,1.0);
  Option<string>  tfregfile("tf","output file for TfReg",OptionMap::optional,"");


  OptionMap::arguments(argc,argv);
  OptionMap::finished();

  Nucleotide<>::dictionary.clear_all();
  Nucleotide<>::dictionary.populate(NucleotideCode::IUPAC);
  
  Nucleotide<int>::dictionary.clear_all();
  Nucleotide<int>::dictionary.populate(NucleotideCode::IUPAC);

  std::ofstream fout;
  fout.open(std::string(output_name.Value+".dat").c_str());

//   std::ofstream fasta;
//   fasta.open(std::string(output_name.Value+".fna").c_str());

  std::ofstream backtrace;
  backtrace.open(std::string(output_name.Value+".backtrace").c_str());

//   std::ofstream network;
//   network.open(std::string(output_name.Value+".net").c_str());
  
  std::ofstream tfreg;
  if (tfregfile.provided()) 
    {
    tfreg.open(tfregfile.Value);
    tfreg << "temperatures #(" <<  std::string(SVNVERSION) << ") ";
    for (std::string arg : OptionMap::all_arguments) {tfreg << arg << " ";} 
    tfreg <<  std::endl;
    }

  FileParser Target(target_name.Value);
  Target.open();
  Target.contig_at_once();
  
  FileParser Genome(genome_name.Value);
  Genome.open();
  Genome.contig_at_once();

  AlignSequence<SmithWatermanBP<> > align;

  align.Record_backtrace=true;
  align.MainSequence.window_size=max_contig;
  align.MainSequence.shift_read=false;
  align.OutputContigMargin=margin;
  align.ReportFile=&fout;
  align.TfRegFile=&tfreg;
  align.BacktraceFile=&backtrace;
  align.Always_backtrace=true;
  align.score_limit(0.0);
  align.Early_break=false;

  align.Target.window_size=0;
  align.Target.shift_read=false;
  align.MainSequence.window_size=0;
  align.MainSequence.shift_read=false;

  align.Minimal_report=false;
  align.set_display_main_top();
  align.Directions=directions.Value;

  unsigned long int Genome_position=0;

  while (!Genome.eof())
    {
    Genome >> align.MainSequence;
    if (align.MainSequence.size() == 0) break;
    align.MainSequence_description=Genome.contig_table()->just_read()->description();
    align.MainSequence_number++;
      
    while (!Target.eof())
      {
      Target >> align.Target;
      if (align.Target.size() == 0) break;
      align.Target_description=Target.contig_table()->just_read()->description();
      align.Target_number++;
      align.Sequence_position=Genome_position-align.MainSequence.size();
 
      align.run_multi_strand_alignment();
      }
    }

  fout.close();
  
  print_cpu();
  return 0;
}

