// $Id: FastaFormat.cpp 967 2017-07-13 12:04:25Z ge $
/// \file FastaFormat.cpp
/// \brief Contains the bodies of the methods of class FastaFormat
///
/// $Revision: 1.6 $
/// \author Gerald Weber <gweberbh@gmail.com>

#include "FastaFormat.h"

using namespace std;

namespace gbc {

  bool FastaFormat::check_file_type(void)
    {
    char cr; bool is_fasta=false;
    rewind();
    file_handle >> cr;
    if (cr=='>') {is_fasta=true;}
    if (is_fasta && Verbose) {cout << "Fasta format detected." << endl;}
    return is_fasta;
    }

  void FastaFormat::prepare_reading(void)
    {
    if (forward_reading) rewind();
    else
      {
      forward_reading=true;
      map_contigs();
      forward_reading=false;
      file_handle.clear();
      file_handle.seekg(-1,std::ios::end);
      if (file_handle.peek() == '\n' && file_handle.tellg() != (std::streampos)0) file_handle.seekg(-1,std::ios_base::cur);
      }
    }


  /// \brief Fasta type mapping
  ///
  /// \attention Only valid nucleotides are mapped. This may affect the counts of Contig_section
  /// begin and end. For example if the file contains a "W" which is not in the dictionary this
  /// is then simply skipped and not counted.
  void FastaFormat::map_contigs(const std::string key)
    {
    Contig_table.clear();
    Contig_mapping=true;
    streampos current=tellg();
    rewind();
    Nucleotide<char> nt;
    while (!eof())
      {
      read(nt);
      }
    if (!Contig_table.empty())
      {
      Contig_table.current()->Stream_section.end(Last_sequence_position.Stream_position); // The last one is at the end
      Contig_table.current()->Contig_section.end(Last_sequence_position.Nucleotide_position);
      Contig_table.current()->Contig_type=string("fasta");
      }
    the_line.clear();
    file_handle.clear(); // clears any eof or fail state set due to the fact that we went until the end of the file
    file_handle.seekg(current,std::ios_base::beg);
    Contig_mapping=false;
    rewind();
    }

 };






