// $Id: GenbankFormat.cpp 967 2017-07-13 12:04:25Z ge $
/// $Revision: 1.11 $
/// \file GenbankFormat.cpp
/// \brief Contains the function bodies of GenbankFormat
///
/// $Revision: 1.11 $
/// \author Gerald Weber <gweberbh@gmail.com>
#include "GenbankFormat.h"

using namespace std;

namespace gbc {

  GenbankFormat::GenbankFormat(void)
    {
    FileFormat::contig_capable=true;
    Genbank_origin=(streampos)0;
    //Contig_table.Ignore_outside=false;
    }


  bool GenbankFormat::check_file_type(void)
    {
    bool is_genbank=false;
    rewind();
    string word;
    file_handle >> word;
    is_genbank=word == string("LOCUS");
    if (is_genbank && Verbose) {cout << "\nGenbank format detected." << endl;}
    return is_genbank;
    }

  void GenbankFormat::prepare_reading(void)
    {
    file_handle.seekg(find_keyword("ORGANISM"),std::ios_base::beg);
    get_next_line(); Organism=the_line.str();
    Features_pos=find_keyword("FEATURES");
    Genbank_origin=find_keyword("ORIGIN");
    if (Verbose) { cout << "ORIGIN found at " << Genbank_origin << endl;}
    goto_origin();
    }

  void GenbankFormat::goto_origin(void)
    {
    file_handle.clear(); //the_line.clear();
    file_handle.seekg(Genbank_origin,std::ios_base::beg);
    if (tellg()!=Genbank_origin)
      {
      std::cerr << "Failed to go to Genbank origin" << std::endl;
      }
    goto_next_line(); get_next_line();
    Last_sequence_position.reset();
    }

  streampos GenbankFormat::find_keyword(const std::string &keyword)
    {
    streampos current=tellg();
    string word; bool found=false;
    while (!eof() && !found)
      {
      file_handle >> word;
      found=word==keyword;
      if (!found) goto_next_line();
      }
    streampos wherefound=tellg();
    file_handle.seekg(current);
    if (found) return wherefound; else return (streampos)0;
    }

  string GenbankFormat::feature_key(void)
    {
    string line=the_line.str().substr(0,20); //looks at the first 20 positions in a line
    int start=line.find_first_not_of(" ");
    if ((start > 0) && (start < 20))
      {
      int end=line.find(" ",start);
      line=line.substr(start,(end-start));
      }
    else line=string();
    return line;
    }

void GenbankFormat::get_section(list<ContigSection> &lcs, string& feature )
  {
  char num[]="0123456789"; unsigned int length=feature.length();
  unsigned int start1=feature.find_first_of(num),        end1=feature.find_first_not_of(num,start1);
  if (start1 > length) return;
  if (end1 > length) end1=length;
  unsigned int start2=feature.find_first_of(num,end1), end2=feature.find_first_not_of(num,start2);
  if (start2 > length) {start2=start1; end2=end1;}
  else {if (end2 > length) end2=length;}
  stringstream line;
  line << feature.substr(start1,end1-start1) << " " << feature.substr(start2,end2-start2) << endl;
  ContigSection sec;
  line >> sec.first >> sec.second;
  if (Verbose) {cout << "s" << flush;}
  lcs.push_back(sec);
  feature.erase(start1,end2-start1);
  }

unsigned int how_many(string target, string& line)
  {
  unsigned int found=0, length=line.length(), last=line.find_first_of(target);
  while (last < length) {found++; last=line.find_first_of(target,last+1);}
  return found;
  }

// See 3.5.3 in http://www.ncbi.nlm.nih.gov/projects/collab/FT/index.html
void GenbankFormat::get_location(list<ContigSection> &lcs, bool &complementary)
    {
    string feature(the_line.str());
    feature.erase(0,20);
    while (how_many("(",feature)!=how_many(")",feature))
      {
      get_next_line(); string feature2 = the_line.str();
      feature2.erase(0,20);
      feature+=feature2;
      if (eof()) {cerr << "File ended while trying to match parenteses" << endl; exit(1);}
      }
    if (feature.find("complement") < feature.length())
      {
      feature.erase(feature.find("complement"),9);
      complementary=true;
      }
    if (feature.find("join(") < feature.length())
      {
      feature.erase(feature.find("join("),5);
      get_section(lcs,feature);
      unsigned int comma=feature.find_first_of(",");
      while(comma < feature.length())
        {
	feature.erase(comma,1);
	get_section(lcs,feature);
        comma=feature.find_first_of(",");
	if (comma == feature.length()) break;
        }
      }
    else
      {
      get_section(lcs,feature);
      }
    }

  void GenbankFormat::map_contigs(const string key)
    {
    Contig_table.clear();
    Contig_mapping=true;
    file_handle.seekg(Features_pos); get_next_line();
    streampos current=tellg();
    get_next_line();
    while (current < Genbank_origin)
      {
      string fkey=feature_key();
      if(fkey!=string() && (fkey==key || key==string()))
        {
        if (Verbose) {cout << "\nFound feature_key " << fkey << " at " << tellg() << flush;}
    	list<ContigSection> lcs;
	bool complementary=false;
	ContigDescription cd;
        cd.Contig_type=string(fkey);
        get_location(lcs,complementary);
	get_next_line();
	int description_count=0;
        while(feature_key()==string())
	  {
          if (Verbose) {cout << "." << flush;}
    	  the_line.seekg((streampos)20);
	  string description;
//	  description =string(the_line.str());
	  the_line >> description;
	  if (description!=string())
            {
	    if (description_count == 0) cd.description(description);
	    else                        cd.next_description(description);
            if (Verbose) {cout << "Description: " << description << endl;}
            description_count++;
	    if (eof()) exit(1);
	    }
	  get_next_line();
	  }
	int join=0;
        while (!lcs.empty())
	  {
	  cd.Contig_section=lcs.front(); lcs.pop_front();
          cd.Complementary=complementary;
	  cd.Join=++join;
          Contig_table.new_contig(cd);
	  }
	if (Verbose) {cout << "\nContigTable size " << Contig_table.size() << endl;}

	}
      else get_next_line();
      current=tellg();
      }
    Contig_table.sort();
    the_line.clear();
    file_handle.clear(); // clears any eof or fail state set due to the fact that we went until the end of the file
    goto_origin();
    Contig_mapping=false;
    if (Verbose) {cout << "\nContig mapping finished" << endl;}
    }

  void GenbankFormat::verify_contig_begin(void)
    {
    Contig_begin=Contig_table.Contig_begin;
    }

};







