// $Id: stdmatrix.h 628 2011-08-31 15:42:16Z ge $
/// \file stdmatrix.h
/// \brief contains the stdmatrix class
///
/// $Revision: 628 $
/// \author Gerald Weber <gweberbh@gmail.com>

#ifndef STDMATRIX_H
#define STDMATRIX_H "$Id: stdmatrix.h 628 2011-08-31 15:42:16Z ge $"

#include <utility>

namespace gbc_exp {
using namespace gbc;

template<class _Tp>
class stdmatrix
  {
  public:
    typedef _Tp  value_type;
    typedef _Tp* pointer_type;
    size_t   Columns, Rows;
    pointer_type Array;
    stdmatrix(void): Columns(1), Rows(1)
      {
      Array=new value_type[1];
      Array[0]=value_type();
      }

  /// \brief Resizes the number of rows and columns of the array.
  ///
  /// The method first checks if the total size of the array changes
  /// and then resizes the valarray if necessary. 
  inline void resize(const size_t& r, const size_t& c)
    {
    if (r*c != Columns*Rows)
      {
      delete[] Array;
      Array=new value_type[r*c];
      Rows=r; Columns=c;
      for(size_t i=0; i < Columns*Rows; ++i) Array[i]=value_type();
      }
    else
      {
      if ((r != Rows) || (c != Columns))
        {Rows=r; Columns=c;}
      }
    }

  /// \brief Read-only access to the (r,c)-th element.
  inline value_type operator() (size_t r, size_t c) const
    {return Array[r*Columns+c];}

  /// \brief Access to the (r,c)-th element.
  inline value_type& operator() (size_t r, size_t c) 
    {return Array[r*Columns+c];}


  /// \brief Provides access to the r-th element of the internal std::valarray
  inline value_type operator[] (size_t r) const 
    {return Array[r];}

  /// \brief Provides access to the r-th element of the internal std::valarray
  inline value_type& operator[] (size_t r) 
    {return Array[r];}
  };

};//ends namespace
#endif