// $Id: zipbuffer.cpp 1069 2019-05-09 15:23:40Z ge $
/// \file zipbuffer
/// \brief testing read of gziped files
///
/// \author Gerald Weber <gweberbh@gmail.com>
/// $Revision: 1.1 $
#include "zipbuffer.h"
#include <iostream>
#include <cstring>

namespace gbc  {
/// filebuffer for handling gziped files transparently
    zipbuffer::zipbuffer(void) 
      {
      zipfile=0;
      }
    
    zipbuffer::zipbuffer(std::string file_name) 
      {
      zipfile=gzopen(file_name.c_str(), "rb"); 
      setg(buffer+putbacksize,buffer+putbacksize,buffer+putbacksize);
      }
    
    void zipbuffer::zipopen(std::string file_name) 
      {
      zipfile=gzopen(file_name.c_str(), "rb"); 
      setg(buffer+putbacksize,buffer+putbacksize,buffer+putbacksize);
      }
    
    void zipbuffer::zipclose(void) 
      {
      gzclose(zipfile);
      }
   
    void zipbuffer::ziprewind(void)
      {
      gzrewind(zipfile);
      setg(buffer+putbacksize,buffer+putbacksize,buffer+putbacksize);
      }

    bool zipbuffer::zipeof(void) {return gzeof(zipfile);}

   std::filebuf::int_type zipbuffer::underflow(void)
      {
      if (gptr() < egptr()) {return *gptr();}
      int numPutback;
      numPutback = gptr() -eback();
      if (numPutback > putbacksize) {numPutback=putbacksize;}
      memcpy(buffer+(putbacksize-numPutback),gptr()-numPutback,numPutback);
      int num;
      num =  gzread(zipfile, buffer+putbacksize, bufferSize-putbacksize);
      if (num <= 0) {return EOF;}
      setg(buffer+(putbacksize-numPutback),buffer+putbacksize,buffer+putbacksize+num);
      return *gptr();
      }
    
    std::filebuf::pos_type zipbuffer::seekpos(std::filebuf::pos_type pos, std::ios::openmode __mode)
      {
      //std::cout << "pos=" << pos;
      setg(buffer+putbacksize,buffer+putbacksize,buffer+putbacksize);
      return gztell(zipfile);
      }
    
    std::filebuf::pos_type zipbuffer::seekoff (std::filebuf::off_type __off, std::ios_base::seekdir __way, std::ios_base::openmode __mode)
      {
      //std::cout << "off=" << __way;
      std::filebuf::pos_type oldpos=gztell(zipfile), newpos;
      if (__way) newpos=gzseek(zipfile,__off,SEEK_CUR); // offset from current position
      else       newpos=gzseek(zipfile,__off,SEEK_SET); // offset from begining of file
      if (oldpos != newpos) setg(buffer+putbacksize,buffer+putbacksize,buffer+putbacksize);
      //std::cout << "off=" << gztell(zipfile);
      int num=egptr()-gptr();
      return newpos-(std::filebuf::pos_type)num;//gztell(zipfile);
      
      }
}

